/*
 * $Header: /cvsroot/gtk2-perl/gtk2-perl-xs/Gtk2-GLExt/gtkglextperl.h,v 1.1 2003/11/16 19:56:35 rwmcfa1 Exp $
 */

#include <gperl.h>
#include <gtk2perl.h>

#include <gdk/gdkgl.h>
#include <gtk/gtkgl.h>
#include <gtkglextperl-autogen.h>
